"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var immutable_1 = require('immutable');
var StatusReporter_1 = require('./StatusReporter');
var actions_1 = require('./actions');
var Set = (function (_super) {
    __extends(Set, _super);
    function Set(items, synchronized) {
        var _this = this;
        if (synchronized === void 0) { synchronized = true; }
        _super.call(this, synchronized);
        this._original = this;
        if (items) {
            this._items = immutable_1.OrderedMap(items.map(function (values) {
                var newItem = _this.constructItem(true, values);
                return [newItem.id, newItem];
            }));
        }
        else {
            this._items = immutable_1.OrderedMap();
        }
    }
    Object.defineProperty(Set.prototype, "original", {
        get: function () {
            return this._original;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Set.prototype, "items", {
        get: function () {
            return this._items;
        },
        enumerable: true,
        configurable: true
    });
    Set.prototype.reduce = function (action) {
        var _this = this;
        if (actions_1.CLAY_DATA_CLEAR.is(this.id, action)) {
            return this._original;
        }
        if (actions_1.CLAY_DATA_RESET.is(this.id, action)) {
            var clone = this.clone();
            clone._status = this._status.setUnmodified();
            clone._items = immutable_1.OrderedMap(action.payload.map(function (values) {
                var newItem = _this.constructItem(true, values);
                return [newItem.id, newItem];
            }));
            clone._original = clone;
            return clone;
        }
        if (actions_1.CLAY_DATA_ADD_ITEM.is(this.id, action)) {
            var clone = this.clone();
            if (this._status.synchronized) {
                clone._status = this._status.setModified();
            }
            var newItem = this.constructItem(false, action.payload);
            clone._items = this._items.set(newItem.id, newItem);
            return clone;
        }
        if (actions_1.CLAY_DATA_REMOVE_ITEM.is(this.id, action)) {
            var newItems_1;
            var target = this._items.get(action.payload);
            if (target.status.added) {
                newItems_1 = this._items.remove(action.payload);
            }
            else {
                newItems_1 = this._items.set(action.payload, target.setRemoved());
            }
            if (immutable_1.is(this._original._items, newItems_1)) {
                return this._original;
            }
            var clone = this.clone();
            if (this._status.synchronized) {
                clone._status = this._status.setModified();
            }
            clone._items = newItems_1;
            return clone;
        }
        var updated = false;
        var newItems = this._items.withMutations(function (i) {
            _this._items.forEach(function (item) {
                if (item.status.removed) {
                    return;
                }
                var newItem = item.reduce(action);
                if (newItem !== item) {
                    updated = true;
                    i.set(newItem.id, newItem);
                }
            });
        });
        if (updated) {
            if (immutable_1.is(this._original._items, newItems)) {
                return this._original;
            }
            var clone = this.clone();
            if (this._status.synchronized) {
                clone._status = this._status.setModified();
            }
            clone._items = newItems;
            return clone;
        }
        return this;
    };
    Set.prototype.clear = function (dispatch) {
        dispatch(actions_1.CLAY_DATA_CLEAR.create.unicast(this.id));
    };
    Set.prototype.add = function (dispatch, values) {
        dispatch(actions_1.CLAY_DATA_ADD_ITEM.create.unicast(this.id, values));
    };
    Set.prototype.remove = function (dispatch, target) {
        dispatch(actions_1.CLAY_DATA_REMOVE_ITEM.create.unicast(this.id, target));
    };
    Set.prototype.refresh = function (dispatch, query, fetcher) {
        var _this = this;
        var fetchPromise = fetcher.add(query);
        return fetchPromise.then(function (result) {
            dispatch(actions_1.CLAY_DATA_RESET.create.unicast(_this.id, result.data || []));
        });
    };
    Set.prototype.sync = function (dispatch, synchronizer) {
        this.appendSynchronizationModels(synchronizer);
    };
    Set.prototype.appendSynchronizationModels = function (synchronizer) {
        this._items.forEach(function (item) { return item.appendSynchronizationModels(synchronizer); });
    };
    Set.prototype.buildInsertModel = function () {
        return this._items.filter(function (item) { return item.status.added; }).map(function (item) { return item.buildInsertModel(); }).toArray();
    };
    Set.prototype.checkValidity = function () {
        if (this._items.some(function (r) { return !r.valid; })) {
            return false;
        }
        return true;
    };
    Set.prototype.cloneOverride = function (clone) {
        _super.prototype.cloneOverride.call(this, clone);
        clone._original = this._original;
        clone._items = this._items;
    };
    return Set;
}(StatusReporter_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Set;
